@@ -31,6 +31,7 @@
 #include "constant.h"
 #include "infinity.h"
 #include "compiler.h"
+#include "order.h"
 
 #include <sstream>
 #include <iostream>
@@ -127,8 +128,9 @@ void add::print_add(const print_context & c, unsigned level, bool latex) const
 	numeric coeff;
 	bool first = true;
 
+	const epvector & sorted_seq = get_sorted_seq();
 	// Then proceed with the remaining factors
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
+	epvector::const_iterator it = sorted_seq.begin(), itend = sorted_seq.end();
 	while (it != itend) {
 		std::stringstream tstream;
 		print_context *tcontext_p;
@@ -695,4 +697,25 @@ ex add::expand(unsigned options) const
 	return (new add(vp, overall_coeff))->setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0));
 }
 
+const epvector & add::get_sorted_seq() const
+{
+	if (seq_sorted.empty() and not seq.empty()) {
+		seq_sorted = epvector(seq.size());
+		partial_sort_copy(seq.begin(), seq.end(),
+				  seq_sorted.begin(), seq_sorted.end(),
+				  print_order_pair());
+	}
+	return expairseq::get_sorted_seq();
+}
+
+ex add::lead_coeff() const {
+	// if sorted_seq was computed before, we don't have to search
+	if (seq_sorted.empty() and not seq.empty()) {
+		return min_element(seq.begin(), seq.end(),
+				print_order_pair())->coeff;
+	} else {
+		return seq_sorted.begin()->coeff;
+	}
+}
+
 } // namespace GiNaC
