@@ -309,6 +309,15 @@ ex expairseq::op(size_t i) const
 	return overall_coeff;
 }
 
+ex expairseq::stable_op(size_t i) const
+{
+	if (i < seq.size()) {
+		return recombine_pair_to_ex(get_sorted_seq()[i]);
+	}
+	GINAC_ASSERT(!overall_coeff.is_equal(default_overall_coeff()));
+	return overall_coeff;
+}
+
 ex expairseq::map(map_function &f) const
 {
 	std::auto_ptr<epvector> v(new epvector);
@@ -392,9 +401,9 @@ bool expairseq::match(const ex & pattern, lst & repl_lst) const
 		bool has_global_wildcard = false;
 		ex global_wildcard;
 		for (size_t i=0; i<pattern.nops(); i++) {
-			if (is_exactly_a<wildcard>(pattern.op(i))) {
+			if (is_exactly_a<wildcard>(pattern.sorted_op(i))) {
 				has_global_wildcard = true;
-				global_wildcard = pattern.op(i);
+				global_wildcard = pattern.sorted_op(i);
 				break;
 			}
 		}
@@ -406,12 +415,12 @@ bool expairseq::match(const ex & pattern, lst & repl_lst) const
 		exvector ops;
 		ops.reserve(nops());
 		for (size_t i=0; i<nops(); i++)
-			ops.push_back(op(i));
+			ops.push_back(stable_op(i));
 
 		// Now, for every term of the pattern, look for a matching term in
 		// the expression and remove the match
 		for (size_t i=0; i<pattern.nops(); i++) {
-			ex p = pattern.op(i);
+			ex p = ex_to<expairseq>(pattern).stable_op(i);
 			if (has_global_wildcard && p.is_equal(global_wildcard))
 				continue;
 			exvector::iterator it = ops.begin(), itend = ops.end();
@@ -492,46 +501,48 @@ int expairseq::compare_same_type(const basic &other) const
 {
 	GINAC_ASSERT(is_a<expairseq>(other));
 	const expairseq &o = static_cast<const expairseq &>(other);
-	
+
 	int cmpval;
-	
+
 	// compare number of elements
 	if (seq.size() != o.seq.size())
 		return (seq.size()<o.seq.size()) ? -1 : 1;
 
+	// compare overall_coeff
+	cmpval = overall_coeff.compare(o.overall_coeff);
+	if (cmpval!=0)
+		return cmpval;
+
 #if EXPAIRSEQ_USE_HASHTAB
 	GINAC_ASSERT(hashtabsize==o.hashtabsize);
 	if (hashtabsize==0) {
 #endif // EXPAIRSEQ_USE_HASHTAB
+
 		epvector::const_iterator cit1 = seq.begin();
 		epvector::const_iterator cit2 = o.seq.begin();
 		epvector::const_iterator last1 = seq.end();
 		epvector::const_iterator last2 = o.seq.end();
 
 		for (; (cit1!=last1)&&(cit2!=last2); ++cit1, ++cit2) {
-			cmpval = (*cit2).compare(*cit1);
+			cmpval = (*cit1).compare(*cit2);
 			if (cmpval!=0) return cmpval;
 		}
 
 		GINAC_ASSERT(cit1==last1);
 		GINAC_ASSERT(cit2==last2);
-	
-		// compare overall_coeff
-		cmpval = overall_coeff.compare(o.overall_coeff);
-		if (cmpval!=0)
-			return cmpval;
-	
+
 		return 0;
+
 #if EXPAIRSEQ_USE_HASHTAB
 	}
-	
+
 	// compare number of elements in each hashtab entry
 	for (unsigned i=0; i<hashtabsize; ++i) {
 		unsigned cursize=hashtab[i].size();
 		if (cursize != o.hashtab[i].size())
 			return (cursize < o.hashtab[i].size()) ? -1 : 1;
 	}
-	
+
 	// compare individual (sorted) hashtab entries
 	for (unsigned i=0; i<hashtabsize; ++i) {
 		unsigned sz = hashtab[i].size();
@@ -549,7 +560,6 @@ int expairseq::compare_same_type(const basic &other) const
 			}
 		}
 	}
-	
 	return 0; // equal
 #endif // EXPAIRSEQ_USE_HASHTAB
 }
@@ -1770,6 +1780,15 @@ std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned option
 	return std::auto_ptr<epvector>(0);
 }
 
+
+const epvector & expairseq::get_sorted_seq() const
+{
+	if (seq_sorted.empty())
+		return seq;
+	else
+		return seq_sorted;
+}
+
 //////////
 // static member variables
 //////////
