@@ -34,12 +34,13 @@
 #include "infinity.h"
 #include "function.h"
 #include "inifcns.h"
+#include "order.h"
 
 #include <iostream>
 #include <vector>
+#include <cmath>
 #include <stdexcept>
 #include <limits>
-#include <cmath>
 #include <sstream>
 #ifdef DO_GINAC_ASSERT
 #  include <typeinfo>
@@ -261,7 +262,8 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 
 	// Separate factors into those with negative numeric exponent
 	// and all others
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
+	const epvector & sorted_seq = get_sorted_seq();
+	epvector::const_iterator it = sorted_seq.begin(), itend = sorted_seq.end();
 	exvector neg_powers, others;
 	while (it != itend) {
 		GINAC_ASSERT(is_exactly_a<numeric>(it->coeff));
@@ -347,11 +349,35 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 		}
 
 	} else {
+		// negative powers are empty
+		if (!latex_tags) {
+			print_overall_coeff(overall_coeff, c, sep, latex_tags);
 
-		print_overall_coeff(overall_coeff, c,
-				latex_tags ? " \\, " : sep, latex_tags);
+			print_exvector(others, c, sep);
+		} else {
+			// Decide which separator to use after the overall
+			// coefficient.
+			// overall coefficient is printed first in the
+			// output, if the first character after the
+			// coefficient is a number, then it is hard to
+			// distinguish these in the latex output:
+			//
+			// sage: e = 2 * 2^(1/3)
+			// sage: print latex(e)
+			// 2 \, 2^{\left(\frac{1}{3}\right)}
+			//
+			// In this case instead of the usual separator \,
+			// we use \cdot
 
-		print_exvector(others, c, sep);
+			std::stringstream tstream;
+			print_latex tcontext(tstream, c.options);
+			print_exvector(others, tcontext, sep);
+			print_overall_coeff(overall_coeff, c,
+					std::isdigit(tstream.peek()) ?
+						" \\cdot " : " \\, ",
+					latex_tags);
+			c.s<<tstream.str();
+		}
 	}
 	if (precedence() <= level){
 		if (latex_tags)
@@ -694,9 +720,13 @@ ex mul::eval(int level) const
 			// XXX: What is the best way to check if the polynomial is a primitive? 
 			numeric c = i->rest.integer_content();
 			const numeric lead_coeff =
-				ex_to<numeric>(ex_to<add>(i->rest).seq.begin()->coeff).div(c);
+				ex_to<numeric>(ex_to<add>(i->rest).\
+						lead_coeff()).div(c);
 			const bool canonicalizable = lead_coeff.is_integer();
 
+			// The following comment is no longer true for pynac.
+			// We use the print order to determine the main variable
+			// This order is not random.
 			// XXX: The main variable is chosen in a random way, so this code 
 			// does NOT transform the term into the canonical form (thus, in some
 			// very unlucky event it can even loop forever). Hopefully the main
@@ -733,6 +763,7 @@ ex mul::eval(int level) const
 			primitive->setflag(status_flags::dynallocated);
 			primitive->clearflag(status_flags::hash_calculated);
 			primitive->overall_coeff = ex_to<numeric>(primitive->overall_coeff).div_dyn(c);
+			primitive->seq_sorted.resize(0);
 			for (epvector::iterator ai = primitive->seq.begin();
 					ai != primitive->seq.end(); ++ai)
 				ai->coeff = ex_to<numeric>(ai->coeff).div_dyn(c);
@@ -979,7 +1010,8 @@ bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, lst &repls,
 			continue;
 		lst newrepls = repls;
 		int newnummatches = nummatches;
-		if (tryfactsubs(e.op(i), pat.op(factor), newnummatches, newrepls)) {
+		if (tryfactsubs(e.op(i), pat.op(factor),
+					newnummatches, newrepls)) {
 			matched[i] = true;
 			if (algebraic_match_mul_with_mul(e, pat, newrepls, factor+1,
 					newnummatches, subsed, matched)) {
@@ -1127,6 +1159,8 @@ ex mul::derivative(const symbol & s) const
 	return (new add(addseq))->setflag(status_flags::dynallocated);
 }
 
+/** Total degree of a mul object.
+ Beware that symbolic exponents are wrapped inside pow objects with 1 as coeff. */
 double mul::total_degree() const
 {
 	if (flags & status_flags::tdegree_calculated) {
@@ -1142,118 +1176,9 @@ double mul::total_degree() const
 	return tdegree;
 }
 
-int mul::compare(const basic& other) const
-{
-	static const tinfo_t function_id = find_tinfo_key("function");
-	static const tinfo_t fderivative_id = find_tinfo_key("fderivative");
-	static const tinfo_t pow_id = find_tinfo_key("power");
-	static const tinfo_t symbol_id = find_tinfo_key("symbol");
-	const tinfo_t typeid_this = tinfo();
-	const tinfo_t typeid_other = other.tinfo();
-	if (typeid_this==typeid_other) {
-		GINAC_ASSERT(typeid(*this)==typeid(other));
-		return compare_same_type(other);
-	} else if (typeid_other == pow_id) {
-		return compare_pow(static_cast<const power&>(other));
-	} else if (typeid_other == symbol_id) {
-		return compare_symbol(static_cast<const symbol&>(other));
-	} else if (typeid_other == function_id ||
-			typeid_other == fderivative_id) {
-		return -1;
-	} else {
-		return (typeid_this<typeid_other ? -1 : 1);
-	}
-}
-
-int mul::compare_symbol(const symbol &other) const
-{
-	int cmpval;
-	double tdeg;
-	tdeg = total_degree();
-	if (tdeg == 1) {
-		cmpval = seq[0].rest.compare(other);
-		if (cmpval != 0) {
-			return cmpval;
-		}
-		cmpval = _ex1.compare(seq[0].coeff);
-		if (cmpval != 0) {
-			return cmpval;
-		}
-		return -1;
-	}
-	return tdeg > 1 ? -1 : 1;
-}
-
-// compare this to a pow object
-// first we compare degrees
-// if equal we compare the first item in the sequence to the base in other
-int mul::compare_pow(const power &other) const
-{
-	double my_deg = total_degree();
-	double odeg;
-	numeric oexp;
-	int cmpval = 0;
-	if (is_a<numeric>(other.exponent)) {
-		numeric oexp = ex_to<numeric>(other.exponent);
-		if (oexp.is_real()) {
-			odeg = oexp.to_double();
-		} else {
-			odeg = std::sqrt(std::pow(oexp.real().to_double(), 2) + 
-					std::pow(oexp.imag().to_double(), 2));
-		}
-		if (odeg != my_deg)
-			return my_deg < odeg ? 1 : -1;
-	} else {
-		cmpval = seq[0].coeff.compare(other.exponent);
-		if (cmpval != 0)
-			return cmpval;
-	}
-	cmpval = seq[0].rest.compare(other.basis);
-	if (cmpval != 0) {
-		return cmpval;
-	}
-	if (seq.size() == 1 && overall_coeff.is_equal(_ex1))
-		return 0;
-	return 1;
-}
-
-
 int mul::compare_same_type(const basic & other) const
 {
-	int cmpval;
-
-	const mul &o = static_cast<const mul &>(other);
-
-	// compare total degrees
-	double deg1 = total_degree();
-	double deg2 = o.total_degree();
-	if (deg1 != deg2)
-		return deg1 < deg2 ? 1 : -1;
-
-	// compare each item in this product to correnponding element in other
-	epvector::const_iterator cit1 = seq.begin();
-	epvector::const_iterator cit2 = o.seq.begin();
-	epvector::const_iterator last1 = seq.end();
-	epvector::const_iterator last2 = o.seq.end();
-
-	for (; (cit1!=last1)&&(cit2!=last2); ++cit1, ++cit2) {
-		cmpval = (*cit1).compare(*cit2);
-		if (cmpval != 0)
-			return cmpval;
-	}
-
-	// compare sizes
-	if (cit1 != last1) 
-		return 1;
-	else if (cit2 != last2)
-		return -1;
-
-	// compare overall_coeff
-	cmpval = overall_coeff.compare(o.overall_coeff);
-	if (cmpval!=0)
-		return cmpval;
-
-	return 0;
+	return inherited::compare_same_type(other);
 }
 
 unsigned mul::return_type() const
@@ -1595,6 +1520,17 @@ ex mul::expand(unsigned options) const
 	}
 }
 
+const epvector & mul::get_sorted_seq() const
+{
+	if (seq_sorted.empty() and not seq.empty()) {
+		seq_sorted = epvector(seq.size());
+		partial_sort_copy(seq.begin(), seq.end(),
+				  seq_sorted.begin(), seq_sorted.end(),
+				  print_order_pair_mul());
+	}
+	return expairseq::get_sorted_seq();
+}
+
   
 //////////
 // new virtual functions which can be overridden by derived classes
