@@ -579,21 +579,24 @@ ex power::eval(int level) const
 				}
 				if (q.is_zero()) {  // the exponent was in the allowed range 0<(n/m)<1
 					if (num_basis->is_rational()) {
-				// call rational_power_parts
-				// for a^b return c,d such that a^b = c*d^b
-				PyObject* restuple = py_funcs.py_rational_power_parts(
-						num_basis->to_pyobject(), num_exponent->to_pyobject());
-				if(!restuple) {
-					throw(std::runtime_error("power::eval, error in rational_power_parts"));
-				}
-				PyObject *newbasis = PyTuple_GetItem(restuple,1);
-				PyObject *ppower = PyTuple_GetItem(restuple,0);
-				const ex t = (new power(newbasis,exponent))->setflag(status_flags::dynallocated | status_flags::evaluated);
-				if (PyObject_IsTrue(PyTuple_GetItem(restuple, 2))) {
-					return t;
-				}
-				return (new mul(t, ppower))->setflag(status_flags::dynallocated | status_flags::evaluated);
-				
+						// call rational_power_parts
+						// for a^b return c,d such that a^b = c*d^b
+						PyObject* basis = num_basis->to_pyobject();
+						PyObject* exponent = num_exponent->to_pyobject();
+						PyObject* restuple = py_funcs.py_rational_power_parts(basis, exponent);
+						if(!restuple) {
+							throw(std::runtime_error("power::eval, error in rational_power_parts"));
+						}
+						PyObject *ppower = PyTuple_GetItem(restuple,0);
+						PyObject *newbasis = PyTuple_GetItem(restuple,1);
+						const bool ppower_equals_one = PyObject_IsTrue(PyTuple_GetItem(restuple, 2));
+						ex result = (new power(newbasis,exponent))->setflag(status_flags::dynallocated | status_flags::evaluated);
+						if (not ppower_equals_one)
+							result = (new mul(result, ppower))->setflag(status_flags::dynallocated | status_flags::evaluated);
+						Py_DECREF(restuple);
+						Py_DECREF(basis);
+						Py_DECREF(exponent);
+						return result;
 					}
 					return this->hold();
 				} else if (r.is_zero()) {
@@ -637,7 +640,8 @@ ex power::eval(int level) const
 		if (num_exponent->is_integer() && is_exactly_a<add>(ebasis)) {
 			numeric icont = ebasis.integer_content();
 			const numeric lead_coeff = 
-				ex_to<numeric>(ex_to<add>(ebasis).seq.begin()->coeff).div(icont);
+				ex_to<numeric>(ex_to<add>(ebasis).\
+						lead_coeff()).div(icont);
 
 			const bool canonicalizable = lead_coeff.is_integer();
 			const bool unit_normal = lead_coeff.is_pos_integer();
@@ -650,6 +654,7 @@ ex power::eval(int level) const
 				addp->setflag(status_flags::dynallocated);
 				addp->clearflag(status_flags::hash_calculated);
 				addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
+				addp->seq_sorted.resize(0);
 				for (epvector::iterator i = addp->seq.begin(); i != addp->seq.end(); ++i)
 					i->coeff = ex_to<numeric>(i->coeff).div_dyn(icont);
 
@@ -675,6 +680,7 @@ ex power::eval(int level) const
 						mulp->setflag(status_flags::dynallocated);
 						mulp->clearflag(status_flags::evaluated);
 						mulp->clearflag(status_flags::hash_calculated);
+						mulp->seq_sorted.resize(0);
 						return (new mul(power(*mulp,exponent),
 						                power(num_coeff,*num_exponent)))->setflag(status_flags::dynallocated);
 					} else {
@@ -685,6 +691,7 @@ ex power::eval(int level) const
 							mulp->setflag(status_flags::dynallocated);
 							mulp->clearflag(status_flags::evaluated);
 							mulp->clearflag(status_flags::hash_calculated);
+							mulp->seq_sorted.resize(0);
 							return (new mul(power(*mulp,exponent),
 							                power(abs(num_coeff),*num_exponent)))->setflag(status_flags::dynallocated);
 						}
@@ -900,39 +907,6 @@ ex power::derivative(const symbol & s) const
 	}
 }
 
-int power::compare(const basic& other) const
-{
-	static const tinfo_t mul_id = find_tinfo_key("mul");
-	static const tinfo_t symbol_id = find_tinfo_key("symbol");
-	static const tinfo_t function_id = find_tinfo_key("function");
-	static const tinfo_t fderivative_id = find_tinfo_key("fderivative");
-	const tinfo_t typeid_this = tinfo();
-	const tinfo_t typeid_other = other.tinfo();
-	if (typeid_this==typeid_other) {
-		GINAC_ASSERT(typeid(*this)==typeid(other));
-		return compare_same_type(other);
-	} else if (typeid_other == mul_id) {
-		return -static_cast<const mul&>(other).compare_pow(*this);
-	} else if (typeid_other == symbol_id) {
-		return compare_symbol(static_cast<const symbol&>(other));
-	} else if (typeid_other == function_id ||
-			typeid_other == fderivative_id) {
-		return -1;
-	} else {
-		return (typeid_this<typeid_other ? -1 : 1);
-	}
-}
-
-int power::compare_symbol(const symbol & other) const
-{
-	int cmpval;
-	cmpval = _ex1.compare(exponent);
-	if (cmpval != 0) {
-		return cmpval;
-	}
-	return basis.compare(other);
-}
-
 int power::compare_same_type(const basic & other) const
 {
 	GINAC_ASSERT(is_exactly_a<power>(other));
@@ -942,11 +916,7 @@ int power::compare_same_type(const basic & other) const
 	if (cmpval)
 		return cmpval;
 	else
-	  // SAGE -- I changed the sign below for consistency with standard
-	  // mathematics and all other math software (except mathematica).
-	  // This makes it so x^5 + x^2 prints correctly instead of 
-	  // as x^2 + x^5.   -- William Stein
-		return -exponent.compare(o.exponent);
+		return exponent.compare(o.exponent);
 }
 
 unsigned power::return_type() const
